#ifndef VOBCLASSES_H
#define VOBCLASSES_H

#include <stdio.h>
#include "includes.h"
#include "Pack.h"


// Here's a class for video Cells (a collection of VOBUs with the same VOB/CELL ID).
class CCell
{
public:
	
	// Construct the Cell, according to the number of cells. If 0, a maximum number corresponding to a 1GB
	// file is used. Quite waseful.
	CCell(int NumPacks = 0);
	// This will allocate the Cell with a number of frame consistent with the File descriptor. It will NOT
	// load the cell from the file.
	CCell(FILE* Fd);
	~CCell();
	void		Initialize(int NumPacks);

	int			m_StartLBA;		// start LBA (sector) of this cell in input or output file.
	int			m_EndLBA;		// start LBA (sector) of the next cell in input or output file.
	ushort		m_VOBID;		// The VOB ID for this cell.
	uchar		m_CELLID;		// The CELL ID for this cell.
	uint		m_StartPTS;		// Start PTS for this cell.
	uint		m_EndPTS;		// End PTS for this cell.
	int			m_NavPackCount;	// How many nav packs in this cell.
	int			m_IFrameCount;	// How many Iframes in this cell.
	int			m_PTSFirstNavPack; // Start PTS of first navpack pack in this cell.
	int			m_LBAOfFistButtonNavPack;// LBA of first navpack with button info.
	int			m_NumFoundAudioStreams;	// How many audio streams were found.
	uint		m_FoundAudioStreams[8]; // Stream numbers of found audio streams.


	bool		m_KeepAudio;	// De we want to keep the audio in the cell?
	bool		m_FoundButtons;	// Are there buttons somewhere in the beginning of the cell?
	bool		m_AdjustSRC;	// Indicates the SRC values should be adjusted in the first I-Frame.
	bool		m_FixSubPicStops; // Enables experimental code that removes "stops" in subpic data.
	int			m_MinButtonSize; // Cells with buttons smaller than this and now forced action are killed.
	int			m_PreviewAllCells;	// If true, the FrameSelectFunction() function will be called on all
	// cells that have more than 1 I-frame.
	
	int			m_NumPacks;		// How many packs are in this cell.
	int			m_NumAllocPacks;// How many packs we have room for.
	int			m_FirstButtonIndex;	// Index of first pack with buttons...
	int			m_LastButtonChange; // Index of last navpack where button info changed.
	ushort		m_MaxSizeX;
	ushort		m_MaxSizeY;
	int			m_AllNops;		// Indicates all button commands are NOPs.
	int			m_ForcedAction;	// Indicates a button has a forced action.
	int			m_AnyLinkNext;	// Indicates at least 1 button has a LinkNextCell or LinkNextPG command.


	uchar		m_HeaderData[10];// 10 bytes of header data, used for first I-frame only.

	CPack*		m_Packs;		// An array of packs.

	uchar		m_Buf[SECTOR_SIZE];	// Our work buffer.
	int*		m_VOBUTable;

	int			m_NumberSubpicUnits[32];	// For each streams, how many SPUs we found.

	// This blanks and empties the cell.
	void		ResetCell();

	// This reads the cell from the specified file. m_LBA is derived from the position in the file.
	// The file position MUST be either at the start of the file, or at the start of a new cell.
	int			ReadFromFile(FILE* InputFile, int &CumulReadLBA, bool IgnoreVideo = false);
	// This goes to the next cell and reads it from the file. ReadFromFile() MUST have been previously
	// called.
	int			ReadNextCellFromFile(FILE* InputFile, int &CumulReadLBA, bool IgnoreVideo = false);

	// This is provided for convenience. It seeks InputFile back to the beginning, and goes through
	// the file looking for the cell with the specified VOB/CELL ID.
	// It returns its LBA or -1 if it can't find it. InputFile is also seeked to the beginning of the cell.
	int			FindThisCell(FILE* InputFile, ushort VOBID, uchar CELLID);

	// This appends the Cell to the output file. We have to provide
	// InputFile because none of the data is kept in memory and therefore has to be read from file.
	// OutputFile has to be open in r+b mode so we can read and write to it.
	int			WriteToFile(FILE* InputFile, FILE* OutputFile, int AdjustPacks = 1);

	// This adds this pack a the specified index. All subsequent packs are moved up. m_NumPacks is incremented.
	// returns 0 if successful.
	int			InsertPack(CPack* ThisPack, int Index);

	// This adds this pack at the end of the cell. m_NumPacks is incremented. returns 0 if successful
	int			AppendPack(CPack* ThisPack);

	// These are self-explanatory. They start searching at CurrentPackIndex+1 or CurrentPackIndex-1
	// depending on the search direction.
	int			FindNextNavPack(int CurrentPackIndex);
	int			FindPrevNavPack(int CurrentPackIndex);
	int			FindNextAudioPackInVOBU(int CurrentPackIndex);
	int			FindNextSubPicPack(int CurrentPackIndex, uchar StreamID = 0);
	int			FindLastSubPicPack(int CurrentPackIndex, uchar StreamID = 0);
	int			FindFirstSubPicPack(int CurrentPackIndex, uchar StreamID = 0);
	// This find the index of the last pack in the next I-frame.
	int			FindNextEndIFrame(int CurrentPackIndex);

	int			CountIFrames(int StartIndex, int EndIndex);
	int			FindThisIFrame(int IFrame, int StartLBA);
	int			FindNextIFrame(int StartLBA);
	int			FindLargestIFrame(int StartLBA);
	int			FindSmallestIFrame(int StartLBA);

	FILE*		m_InputImage;

	// Loads a pack data from InputFile into m_Buf.
	int			LoadPack(FILE* InputFile, int PackIndex);
	int			WritePack(FILE* OutputFile, int PackIndex);

	// This resets the cell and copies the first I-Frame from SrcCell, along with its navpack. LEGACY
	int			CopyFirstIFrameNoAudio(CCell* SrcCell);
	int			CopyFirstIFrameWithAudio(CCell* SrcCell);

	// Resets the cell, copies the SrcIFrame-th I-Frame from SrcCell, along with its navpack.
	// If SrcIFrame is -1, the largest I-frame in the cell is selected.
	// Returns 0 if all went well, -1 if there was a problem, -2 if the user decided to keep this cell animated.
	// This might happen via the use of the FrameSelectFunction() pointer.
	int			CopySingleIFrame(CCell* SrcCell, int SrcIFrame, bool KeepAudio);

private:
	// These are helper functions. 
	// This write the pack with index PackIndex at the end of OutputFile. It makes all the necessary
	// adjustments to the pack pointers, depending on whether it's a NavPack, an I-Frame etc...
	int			AdjustAndWritePack(FILE* OutputFile, int PackIndex);

	int			FinalCellAdjust(FILE* InputFile, FILE* OutputFile);

	void		UpdateVOBUTable();
};

#endif