#ifndef CPACK_H
#define CPACK_H
#include "includes.h"

typedef struct
{
	ushort	m_startx;
	ushort	m_starty;
	ushort	m_sizex;
	ushort	m_sizey;
	uchar	m_up;		// Button to go to when pressing up.
	uchar	m_down;		// Button to go to when pressing down.
	uchar	m_left;		// Button to go to when pressing left.
	uchar	m_right;	// Button to go to when pressing right.
	uchar	m_auto;		// Autoaction flag.
} ButtonInfo;

// Here's a class for any kind of pack in a VOB file.
class CPack
{
public:
	CPack();
	~CPack();

	// There would be ways to minimize the size of this structure, as there might be cases where we have
	// to allocate quite a few.
	uint		m_InputLBA;		// LBA (sector) in input file.
	PackType	m_PackType;		// Nav, video, audio or subpic pack.
	uchar		m_StreamID;		// Stream ID for audio or subpics.
	FrameType	m_FrameType;	// Video frame type. I, P or B.
	IFrameType	m_IFrameType;	// Only for I-frames: Start of frame, middle, or last LBA of frame.
	char		m_IFrameTooFull;	// Only for I-frames: indicates we can't add anything to the I-Frame.
	short		m_IFrameSize;	// Size of I-Frame in sectors.
	uint		m_SRCL;			// System Reference Clock 4 lowest bytes.
	ushort		m_SRCH;			// System Reference Clock 2 highest bytes.
	uchar		m_NumButtons;	// Number of buttons for a navpack. Also used for subpic packs.


	// Given a buffer of data from a video pack, this will detect which type of video frame we have.
	void		FindPackType(uchar* Buffer, ushort &VOBID, uchar &CELLID);
	// Given a buffer of data, this will detect which kind of pack this is. If AspectRatio
	// is non zero, the aspect ratio code is returned. 1 for 1:1, 2 for 4:3, 3 for 16:9
	// and 4 for 2.21:1. 
	void		FindFrameType(uchar* Buffer, int* Offset= 0, uchar* AspectRatio = 0,
		int* CodedWidth = 0, int* CodedHeight = 0);

	// This should only be called on an I-frame. It checks for a picture extension
	// Returns -1 if not I-frame, returns 1 if MPEG2, 0 if not.
	int			CheckMPEG2(uchar* Buffer);

	// Use this in a frame that has a GOP header (typically the first I-frame of a VOBU) to determine whether
	// there are closed-captions.
	int			FindClosedCaptions(uchar* Buffer);

	// Finds the offset to the true end of an I-frame, in the mpeg data.
	int			FindEndFrameOffset(uchar* Buffer);

	int			FindSubpicLengthAndOffset(uchar* Buffer, ushort &Length, ushort &Offset, ushort &TrueOffset);

	// This trims the end of an I-Frame, putting an end_of_sequence header and adding a padding stream.
	void		TrimIFrame(uchar* Buf);

	// This adjusts the data in the GOP header of the I-frame. It resets the Closed and Broken GOP flags,
	// and it sets the temporal sequence number to 0.
	void		CloseGOP(uchar* Buffer);

	// Experimental: Removes a timed stop command in the last pack of a subpic stream.
	int			RemoveSubpicStop(uchar* Buffer, uchar* Buffer2, ushort TrueOffset, ushort Offset = 0);

	void		FindMaxButtonSize(uchar* Buffer, ushort &MaxSizeX, ushort &MaxSizeY, int &AllNops,
		int &ForcedAction, int &AnyNext);

	// This is not well typecast. The function returns startx, starty, sizex and sizey for each button, stored
	// in the array AllInfos.
	void		GetButtonInfo(uchar* Buffer, ButtonInfo* AllInfos);

	// This only works on navpacks. Returns an 8-byte array of color schemes. Colors for e2-e1-p-b + contrast for e2-e1-p-b
	// times 2 (one for selection, one for action). That's (2+2) + (2+2); Scheme = 0 for the first one.
	uchar*		GetColorScheme(uchar* Buffer, int Scheme);
};

// A small class to help with manipulating time in the DVD time format (eek!).
class CDVDTime
{
public:
	CDVDTime();
	~CDVDTime();

	CDVDTime(int PTSDuration);	// This initializes it with the duration in PTS ticks (90kHz).

	int		m_hours;
	int		m_minutes;
	int		m_seconds;
	int		m_frames;
	VFormat	m_format;

	// Read 4 bytes from Buffer and converts to a time. Uses the GetInt() function for
	// endian purposes.
	void	ReadFromBuffer(uchar* Buffer);
	// Writes the time back to Buffer (4 bytes), uses PutInt().
	void	WriteToBuffer(uchar* Buffer);

	// Time values are supposed to be positive here!
	CDVDTime	&operator += ( CDVDTime & );
	CDVDTime	&operator -= ( CDVDTime & );
	bool	operator < ( CDVDTime & );

	float	ConvertToSeconds();

	bool	IsZero();

};

// Utility functions...
CompositeSRC ParseSRC(uchar* NavPack);							// From uchar* -> CompositeSRC
void WriteSRC(uchar* NavPack, CompositeSRC SRC);				// CompositeSRC -> uchar*.
void FracToSrc(CompositeSRC CSRC, uint &SRCL, ushort &SRCH);	// From CompositeSRC -> SRCL / SRCH
void SrcToFrac(uint SRCL, ushort SRCH, CompositeSRC &CSRC);		// From SRCL / SRCH -> CompositeSRC
void AddSRC(CompositeSRC &SRC, CompositeSRC SRC2);				// Add SRC += SRC2;
void IncSRC(CompositeSRC &SRC);									// Increment SRC by 1 pack slot.
bool SRC1GESRC2(CompositeSRC SRC1, CompositeSRC SRC2);			// SRC1 >= SRC2
uint ParsePTS(uchar* PTSIn);									// From 5 byte PTS -> uint;
// For PTSPreset, use 
//   - 0x00 for no PTS, 
//   - 0x20 for PTS only, 
//   - 0x30 for PTS part of PTS+DTS,
//   - 0x10 for DTS part of PTS+DTS.
void BuildPTS(uchar* PTSOut, uint PTSIn, uchar PTSPreset);		// From uint -> 5Byte PTS.


// This subclass provides better access to navpack data...
class CNavPack : public CPack
{
public:
	uchar*		m_Buffer;

	// Get the VCID.
	void		GetVCID(ushort &VOBID, uchar &CELLID);
	// Find sector offset to next nav pack.
	uint		GetOffsetToNextNavPack();
	// Get the number of buttons.
	uchar		GetNumButtons();
	// Get highlight status.
	ushort		GetHighlight();
	// Get the command for a specific button.
	void		GetButtonCommand(uchar Button, uchar* Command);
	// Get the size of a specific button
	void		GetButtonSize(uchar Button, ushort &width, ushort &height);

};
#endif